;******************************************************************************
;                                                                             *
;    Filename: main.asm                                                       *
;    Date:                                                                    *
;    File Version:                                                            *
;                                                                             *
;    Author:                                                                  *
;    Company:                                                                 *
;                                                                             * 
;******************************************************************************

	LIST P=18F2550, F=INHX32	;directive to define processor
	#include <P18F2550.INC>	;processor specific variable definitions

;******************************************************************************
;Configuration bits

;   Oscillator Selection bits:
	CONFIG  FOSC = INTOSCIO_EC ; Internal oscillator, port function on RA6, EC used by USB (INTIO)
;   MCLR Pin Enable bit:
	CONFIG	MCLRE = OFF ; RE3 input pin enabled, MCLR pin disabled
;   Single-Supply ICSP Enable bit:
	CONFIG	LVP = OFF ;Single-Supply ICSP disabled
	;This shoud be disabled or PORTB<5:4> behave strangely

;   System Clock Postscaler Selection bits:
;	CONFIG	CPUDIV = OSC1_PLL2 ;[Primary Oscillator Src: /1][96 MHz PLL Src: /2]
;   Fail-Safe Clock Monitor Enable bit:
	CONFIG	FCMEN = OFF ;Fail-Safe Clock Monitor disabled
;   Internal/External Oscillator Switchover bit:
	CONFIG	IESO = OFF ;Oscillator Switchover mode disabled

;   Power-up Timer Enable bit/Brown-out Reset Enable bits
	CONFIG	PWRT = OFF,BOR = OFF
;   Brown-out Reset Voltage bits:
	CONFIG BORV = 0 ;Maximum setting

;   USB Voltage Regulator Enable bit:
	CONFIG	VREGEN = OFF ;USB voltage regulator disabled
;   Watchdog Timer Enable bit:
	CONFIG	WDT = OFF ;WDT disabled (control is placed on the SWDTEN bit)
;   CCP2 MUX bit:
	CONFIG	CCP2MX = OFF ;CCP2 input/output is multiplexed with RB3
;   PORTB A/D Enable bit:
	CONFIG	PBADEN = OFF ;PORTB<4:0> pins are configured as digital I/O on Reset
;   Low-Power Timer 1 Oscillator Enable bit:
	CONFIG	LPT1OSC = OFF ;Timer1 configured for higher power operation

;   Stack Full/Underflow Reset Enable bit:
	CONFIG	STVREN = OFF ;Stack full/underflow will not cause Reset
;   Extended Instruction Set Enable bit:
	CONFIG	XINST = OFF ;Instruction set extension and Indexed Addressing mode disabled (Legacy mode)

;   Background Debugger Enable bit:
	CONFIG	DEBUG = OFF ;Background debugger disabled, RB6 and RB7 configured as general purpose I/O pins

;   Protection bits:
	CONFIG	CPB = OFF,CP0 = OFF,CP1 = OFF,CP2 = OFF,CP3 = OFF,CPD = OFF
	CONFIG	WRT0 = OFF,WRT1 = OFF,WRT2 = OFF,WRT3 = OFF,WRTC = OFF,WRTB = OFF,WRTD = OFF
	CONFIG	EBTR0 = OFF,EBTR1 = OFF,EBTR2 = OFF,EBTR3 = OFF,EBTRB = OFF

;******************************************************************************

	#include <tetris.inc>
	
;******************************************************************************

;Variable definitions

		UDATA_ACS

; Drivers

	GLOBAL BUTTONS_STATUS
BUTTONS_STATUS	RES	1

	GLOBAL MATRIX_FRAMEBUFFER
MATRIX_FRAMEBUFFER	RES	0x10 ; (=16 bytes)

	GLOBAL SEVENSEG_FRAMEBUFFER
SEVENSEG_FRAMEBUFFER	RES	0x04 ; (=4 bytes)

; Timer 0 ISR

FSR1_TEMP	RES 2 ; to push/pop FSR1 in timer 0 IRQ handler

MATRIX_SELECT_BIT RES 1 ; Current row being drawn
MATRIX_SELECT	  RES 1 ; Current row number being drawn

SSEG_SELECT	RES 1 ; Current 7 segment display being drawn

;******************************************************************************
;Reset vector
; This code will start executing when a reset occurs.

RESET_VECTOR	CODE	0x0000

	goto	Main		;go to start of main code

;******************************************************************************
;High priority interrupt vector
; This code will start executing when a high priority interrupt occurs or
; when any interrupt occurs if interrupt priorities are not enabled.

HI_INT_VECTOR	CODE	0x0008

	btfsc INTCON,TMR0IF
	call Timer0IRQHandler

	btfsc PIR1,TMR2IF
	call Timer2IRQHandler
	
	retfie	FAST

;******************************************************************************
; Rest of the code

		CODE

;******************************************************************************

Timer0IRQHandler

	; Clear interrupt flag
	bcf	INTCON,TMR0IF

	; Push FSR1
	movff FSR1L,FSR1_TEMP+0
	movff FSR1H,FSR1_TEMP+1
	
	; First, LED matrix
	; -----------------
	
	incf	MATRIX_SELECT,W ; increment
	andlw	0x07 ; clamp to 0~7
	movwf	MATRIX_SELECT ; and save

LEDM_SER EQU 0 ; serial data
LEDM_RCK EQU 1 ; refresh outputs
LEDM_SCK EQU 2 ; serial clock
LEDM_REGISTER EQU PORTA

	; Search start of framebuffer data that has to be sent
	lfsr	FSR1,MATRIX_FRAMEBUFFER
	addlw	8
	movf	PLUSW1,W ; Add 8 AND THEN get first byte
	
	; Clear display
	bsf	LEDM_REGISTER,LEDM_SER ; led off
	    bsf	LEDM_REGISTER,LEDM_SCK ; CLK
	    bcf	LEDM_REGISTER,LEDM_SCK
	    bsf	LEDM_REGISTER,LEDM_SCK ; CLK
	    bcf	LEDM_REGISTER,LEDM_SCK
	    bsf	LEDM_REGISTER,LEDM_SCK ; CLK
	    bcf	LEDM_REGISTER,LEDM_SCK
	    bsf	LEDM_REGISTER,LEDM_SCK ; CLK
	    bcf	LEDM_REGISTER,LEDM_SCK
	    bsf	LEDM_REGISTER,LEDM_SCK ; CLK
	    bcf	LEDM_REGISTER,LEDM_SCK
	    bsf	LEDM_REGISTER,LEDM_SCK ; CLK
	    bcf	LEDM_REGISTER,LEDM_SCK
	    bsf	LEDM_REGISTER,LEDM_SCK ; CLK
	    bcf	LEDM_REGISTER,LEDM_SCK
	    bsf	LEDM_REGISTER,LEDM_SCK ; CLK
	    bcf	LEDM_REGISTER,LEDM_SCK
	    bsf	LEDM_REGISTER,LEDM_SCK ; CLK
	    bcf	LEDM_REGISTER,LEDM_SCK
	    bsf	LEDM_REGISTER,LEDM_SCK ; CLK
	    bcf	LEDM_REGISTER,LEDM_SCK
	    bsf	LEDM_REGISTER,LEDM_SCK ; CLK
	    bcf	LEDM_REGISTER,LEDM_SCK
	    bsf	LEDM_REGISTER,LEDM_SCK ; CLK
	    bcf	LEDM_REGISTER,LEDM_SCK
	    bsf	LEDM_REGISTER,LEDM_SCK ; CLK
	    bcf	LEDM_REGISTER,LEDM_SCK
	    bsf	LEDM_REGISTER,LEDM_SCK ; CLK
	    bcf	LEDM_REGISTER,LEDM_SCK
	    bsf	LEDM_REGISTER,LEDM_SCK ; CLK
	    bcf	LEDM_REGISTER,LEDM_SCK
	    bsf	LEDM_REGISTER,LEDM_SCK ; CLK
	    bcf	LEDM_REGISTER,LEDM_SCK
	bsf	LEDM_REGISTER,LEDM_RCK ; Refresh outputs
	bcf	LEDM_REGISTER,LEDM_RCK
		
		; Negative logic...

		bsf	LEDM_REGISTER,LEDM_SER ; led off
		btfsc	WREG,7
		bcf	LEDM_REGISTER,LEDM_SER ; if set, led on
		bsf	LEDM_REGISTER,LEDM_SCK ; CLK
		bcf	LEDM_REGISTER,LEDM_SCK

		bsf	LEDM_REGISTER,LEDM_SER ; led off
		btfsc	WREG,6
		bcf	LEDM_REGISTER,LEDM_SER ; if set, led on
		bsf	LEDM_REGISTER,LEDM_SCK ; CLK
		bcf	LEDM_REGISTER,LEDM_SCK

		bsf	LEDM_REGISTER,LEDM_SER ; led off
		btfsc	WREG,5
		bcf	LEDM_REGISTER,LEDM_SER ; if set, led on
		bsf	LEDM_REGISTER,LEDM_SCK ; CLK
		bcf	LEDM_REGISTER,LEDM_SCK

		bsf	LEDM_REGISTER,LEDM_SER ; led off
		btfsc	WREG,4
		bcf	LEDM_REGISTER,LEDM_SER ; if set, led on
		bsf	LEDM_REGISTER,LEDM_SCK ; CLK
		bcf	LEDM_REGISTER,LEDM_SCK

		bsf	LEDM_REGISTER,LEDM_SER ; led off
		btfsc	WREG,3
		bcf	LEDM_REGISTER,LEDM_SER ; if set, led on
		bsf	LEDM_REGISTER,LEDM_SCK ; CLK
		bcf	LEDM_REGISTER,LEDM_SCK

		bsf	LEDM_REGISTER,LEDM_SER ; led off
		btfsc	WREG,2
		bcf	LEDM_REGISTER,LEDM_SER ; if set, led on
		bsf	LEDM_REGISTER,LEDM_SCK ; CLK
		bcf	LEDM_REGISTER,LEDM_SCK

		bsf	LEDM_REGISTER,LEDM_SER ; led off
		btfsc	WREG,1
		bcf	LEDM_REGISTER,LEDM_SER ; if set, led on
		bsf	LEDM_REGISTER,LEDM_SCK ; CLK
		bcf	LEDM_REGISTER,LEDM_SCK

		bsf	LEDM_REGISTER,LEDM_SER ; led off
		btfsc	WREG,0
		bcf	LEDM_REGISTER,LEDM_SER ; if set, led on
		bsf	LEDM_REGISTER,LEDM_SCK ; CLK
		bcf	LEDM_REGISTER,LEDM_SCK

	
	movf	MATRIX_SELECT,W
	movf	PLUSW1,W ; Get second byte
	
		; Negative logic...

		bsf	LEDM_REGISTER,LEDM_SER ; led off
		btfsc	WREG,7
		bcf	LEDM_REGISTER,LEDM_SER ; if set, led on
		bsf	LEDM_REGISTER,LEDM_SCK ; CLK
		bcf	LEDM_REGISTER,LEDM_SCK

		bsf	LEDM_REGISTER,LEDM_SER ; led off
		btfsc	WREG,6
		bcf	LEDM_REGISTER,LEDM_SER ; if set, led on
		bsf	LEDM_REGISTER,LEDM_SCK ; CLK
		bcf	LEDM_REGISTER,LEDM_SCK

		bsf	LEDM_REGISTER,LEDM_SER ; led off
		btfsc	WREG,5
		bcf	LEDM_REGISTER,LEDM_SER ; if set, led on
		bsf	LEDM_REGISTER,LEDM_SCK ; CLK
		bcf	LEDM_REGISTER,LEDM_SCK

		bsf	LEDM_REGISTER,LEDM_SER ; led off
		btfsc	WREG,4
		bcf	LEDM_REGISTER,LEDM_SER ; if set, led on
		bsf	LEDM_REGISTER,LEDM_SCK ; CLK
		bcf	LEDM_REGISTER,LEDM_SCK

		bsf	LEDM_REGISTER,LEDM_SER ; led off
		btfsc	WREG,3
		bcf	LEDM_REGISTER,LEDM_SER ; if set, led on
		bsf	LEDM_REGISTER,LEDM_SCK ; CLK
		bcf	LEDM_REGISTER,LEDM_SCK

		bsf	LEDM_REGISTER,LEDM_SER ; led off
		btfsc	WREG,2
		bcf	LEDM_REGISTER,LEDM_SER ; if set, led on
		bsf	LEDM_REGISTER,LEDM_SCK ; CLK
		bcf	LEDM_REGISTER,LEDM_SCK

		bsf	LEDM_REGISTER,LEDM_SER ; led off
		btfsc	WREG,1
		bcf	LEDM_REGISTER,LEDM_SER ; if set, led on
		bsf	LEDM_REGISTER,LEDM_SCK ; CLK
		bcf	LEDM_REGISTER,LEDM_SCK

		bsf	LEDM_REGISTER,LEDM_SER ; led off
		btfsc	WREG,0
		bcf	LEDM_REGISTER,LEDM_SER ; if set, led on
		bsf	LEDM_REGISTER,LEDM_SCK ; CLK
		bcf	LEDM_REGISTER,LEDM_SCK
		
	; Get current line	
	movf	MATRIX_SELECT_BIT,W ; Get line
	rlncf	WREG ; rotation without carry will preserve the selection bit.
	movwf	MATRIX_SELECT_BIT ; Save current line
	movwf	PORTB ; Update line
		
		; Now, refresh registers
	
		bsf	LEDM_REGISTER,LEDM_RCK ; Refresh outputs
		bcf	LEDM_REGISTER,LEDM_RCK
		
	; Now, 7 segment displays -> negative logic!
	; -----------------------
	
	incf	SSEG_SELECT,W ; increment
	andlw	0x03 ; clamp to 0~3
	movwf	SSEG_SELECT ; and save
	
	; Search start of framebuffer data that has to be sent
	lfsr	FSR1,SEVENSEG_FRAMEBUFFER
	movf	PLUSW1,W

SEVENSEG_SER EQU 2 ; serial data
SEVENSEG_RCK EQU 1 ; refresh outputs
SEVENSEG_SCK EQU 0 ; serial clock
SEVENSEG_REGISTER EQU PORTC
	
	; Clear display
	bcf	SEVENSEG_REGISTER,SEVENSEG_SER ; led on
	    bsf	SEVENSEG_REGISTER,SEVENSEG_SCK ; CLK
	    bcf	SEVENSEG_REGISTER,SEVENSEG_SCK
	    bsf	SEVENSEG_REGISTER,SEVENSEG_SCK ; CLK
	    bcf	SEVENSEG_REGISTER,SEVENSEG_SCK
	    bsf	SEVENSEG_REGISTER,SEVENSEG_SCK ; CLK
	    bcf	SEVENSEG_REGISTER,SEVENSEG_SCK
	    bsf	SEVENSEG_REGISTER,SEVENSEG_SCK ; CLK
	    bcf	SEVENSEG_REGISTER,SEVENSEG_SCK
	    bsf	SEVENSEG_REGISTER,SEVENSEG_SCK ; CLK
	    bcf	SEVENSEG_REGISTER,SEVENSEG_SCK
	    bsf	SEVENSEG_REGISTER,SEVENSEG_SCK ; CLK
	    bcf	SEVENSEG_REGISTER,SEVENSEG_SCK
	    bsf	SEVENSEG_REGISTER,SEVENSEG_SCK ; CLK
	    bcf	SEVENSEG_REGISTER,SEVENSEG_SCK
	    bsf	SEVENSEG_REGISTER,SEVENSEG_SCK ; CLK
	    bcf	SEVENSEG_REGISTER,SEVENSEG_SCK
	bsf	SEVENSEG_REGISTER,SEVENSEG_RCK ; Refresh outputs
	bcf	SEVENSEG_REGISTER,SEVENSEG_RCK
	    
		bsf	SEVENSEG_REGISTER,SEVENSEG_SER ; led off
		btfss	WREG,0
		bcf	SEVENSEG_REGISTER,SEVENSEG_SER ; if set, led on
		bsf	SEVENSEG_REGISTER,SEVENSEG_SCK ; CLK
		bcf	SEVENSEG_REGISTER,SEVENSEG_SCK

		bsf	SEVENSEG_REGISTER,SEVENSEG_SER ; led off
		btfss	WREG,1
		bcf	SEVENSEG_REGISTER,SEVENSEG_SER ; if set, led on
		bsf	SEVENSEG_REGISTER,SEVENSEG_SCK ; CLK
		bcf	SEVENSEG_REGISTER,SEVENSEG_SCK

		bsf	SEVENSEG_REGISTER,SEVENSEG_SER ; led off
		btfss	WREG,2
		bcf	SEVENSEG_REGISTER,SEVENSEG_SER ; if set, led on
		bsf	SEVENSEG_REGISTER,SEVENSEG_SCK ; CLK
		bcf	SEVENSEG_REGISTER,SEVENSEG_SCK

		bsf	SEVENSEG_REGISTER,SEVENSEG_SER ; led off
		btfss	WREG,3
		bcf	SEVENSEG_REGISTER,SEVENSEG_SER ; if set, led on
		bsf	SEVENSEG_REGISTER,SEVENSEG_SCK ; CLK
		bcf	SEVENSEG_REGISTER,SEVENSEG_SCK

		bsf	SEVENSEG_REGISTER,SEVENSEG_SER ; led off
		btfss	WREG,4
		bcf	SEVENSEG_REGISTER,SEVENSEG_SER ; if set, led on
		bsf	SEVENSEG_REGISTER,SEVENSEG_SCK ; CLK
		bcf	SEVENSEG_REGISTER,SEVENSEG_SCK

		bsf	SEVENSEG_REGISTER,SEVENSEG_SER ; led off
		btfss	WREG,5
		bcf	SEVENSEG_REGISTER,SEVENSEG_SER ; if set, led on
		bsf	SEVENSEG_REGISTER,SEVENSEG_SCK ; CLK
		bcf	SEVENSEG_REGISTER,SEVENSEG_SCK

		bsf	SEVENSEG_REGISTER,SEVENSEG_SER ; led off
		btfss	WREG,6
		bcf	SEVENSEG_REGISTER,SEVENSEG_SER ; if set, led on
		bsf	SEVENSEG_REGISTER,SEVENSEG_SCK ; CLK
		bcf	SEVENSEG_REGISTER,SEVENSEG_SCK

		bsf	SEVENSEG_REGISTER,SEVENSEG_SER ; led off
		btfss	WREG,7
		bcf	SEVENSEG_REGISTER,SEVENSEG_SER ; if set, led on
		bsf	SEVENSEG_REGISTER,SEVENSEG_SCK ; CLK
		bcf	SEVENSEG_REGISTER,SEVENSEG_SCK
	
	; Get current display	
	movf	SSEG_SELECT,W
	
	tstfsz  WREG
	bra	NotZero
	; Zero
	bsf	PORTA,5
	;bcf	PORTA,6
	;bcf	PORTC,6
	bcf	PORTC,7

	bra	End7SegBranch
NotZero
	decfsz	WREG
	bra	NotOne
	; One
	bcf	PORTA,5
	bsf	PORTA,6
	;bcf	PORTC,6
	;bcf	PORTC,7
	
	bra	End7SegBranch
NotOne
	decfsz	WREG
	bra	NotTwo
	; Two
	;bcf	PORTA,5
	bcf	PORTA,6
	bsf	PORTC,6
	;bcf	PORTC,7
	
	bra	End7SegBranch
NotTwo
	; Three
	;bcf	PORTA,5
	;bcf	PORTA,6
	bcf	PORTC,6
	bsf	PORTC,7

End7SegBranch
	
		bsf	SEVENSEG_REGISTER,SEVENSEG_RCK ; Refresh outputs
		bcf	SEVENSEG_REGISTER,SEVENSEG_RCK
	
	; Exit
	; ----
	
	; Pop FSR1
	movff FSR1_TEMP+0,FSR1L
	movff FSR1_TEMP+1,FSR1H

	return

;******************************************************************************

Timer2IRQHandler
	
	;Clear interrupt flag
	bcf	PIR1,TMR2IF
	
	;bsf	INTCON,GIEH ; Allow timer 0 to interrupt here -> NO
	
	call	TIMER_Tetris
	
	return

;******************************************************************************
;Start of main program

Main:

	; Disable interrupts
	clrf INTCON
	bcf RCON,IPEN ; disable priorities

	bsf OSCCON,IRCF2
	bsf OSCCON,IRCF1
	bcf OSCCON,IRCF0 ; Set internal clock to 8 MHz
	
	; To use pins RC4 and RC5 as digital inputs, the USB module must be disabled 
	; (UCON<3> = 0) and the on-chip USB transceiver must be disabled (UCFG<3> = 1).
	clrf WREG
	movwf UCON
	movlw 0x08
	movwf UCFG
	
	; Make all pins are digital I/O
	movlw 0x0F
	movwf ADCON1

	clrf PORTA ; clear PORTA
	movlw 0x00
	movwf TRISA ; configure PORTA<6:0> as outputs.
	
	clrf PORTB ; clear PORTB
	movlw 0x00
	movwf TRISB ; configure PORTB<7:0> as outputs.

	clrf PORTC ; clear PORTC
	movlw 0x00 
	movwf TRISC ; PORTC<7:6> and PORTC<2:0> as outputs.
	; PORTC<4> and PORT<5> are inputs only.
	; PORTC<3> doesn't exist. PORTC<5> unused.
	   
	; Clear variables
	; ---------------
	
	movlw	0x7F ; (=~80) in the first iteration it will change to 0xFE
	movwf	MATRIX_SELECT_BIT ; (=~01)
	
	movlw   0x07
	movwf	MATRIX_SELECT ; in the first iteration it will change to 0x00
	
	movlw	0x03
	movwf	SSEG_SELECT ; in the first iteration it will change to 0x00
	
	; Init drivers
	; ------------
	
	clrf	BUTTONS_STATUS
	
	;Clear frame
	movlw 0x10 ; (=16)
	lfsr FSR0,MATRIX_FRAMEBUFFER
ClearMore1
	clrf POSTINC0	 	; MATRIX_FRAMEBUFFER[FSR0++] = 0
	decfsz WREG,W
	bra ClearMore1
	
	movlw 0x04 ; (=4)
	lfsr FSR0,SEVENSEG_FRAMEBUFFER
ClearMore2
	clrf POSTINC0	 	; SEVENSEG_FRAMEBUFFER[FSR0++] = 0
	decfsz WREG,W
	bra ClearMore2
	
	; Init timers
	; -----------
	
	; Init timer 0 
	movlw (1<<TMR0ON)|(1<<T08BIT)|(0<<T0CS)|(0<<T0SE)|(0<<PSA)|(0x2)
	; ON | 8 bit | Instruction cycle clock | Low to high | Preescaler ON | 1:8
	movwf	T0CON ; Configure timer 0
	clrf	TMR0 ; Clear the TMR0 register
	bcf	INTCON,TMR0IF ; clear interrupt flag
	bsf	INTCON,TMR0IE ; enable timer 0 interrupt

	; Init timer 2
	movlw (1<<TMR2ON)| (0x2<<0) |(0x3)<<3;
	; ON | Prescaler 1:16 | Postscaler 1:4
	; ((8MHz/4) / (16 * 4)) / 256 = 122 Hz
	movwf T2CON ; Configure timer 2
	movlw 0xFF
	movwf PR2 ; Postscaler count on TMR2 = PR2 = 0xFF
	clrf TMR2 ; Clear the TMR2 register
	bcf	PIR1,TMR2IF ; clear interrupt flag
	bsf	PIE1,TMR2IE ; enable timer 2 interrupt
	
	; Enable Global interrupts
	bsf INTCON,GIEH ;enable all unmasked interrupts - Timer 0
	bsf INTCON,GIEL ;enable all unmasked peripheral interrupts - Timer 1 and 2
	
	; Global initialization done
	; --------------------------
	
	call SETUP_Tetris
	
	call MAIN_Tetris
	
LoopForever ; Shouldn't reach this point or it will reset the controller.
	reset
	bra LoopForever

;******************************************************************************
	
BTN_CLK EQU 4
BTN_NLOAD EQU 3
BTN_REGISTER EQU PORTA
BTN_OUT EQU 4
BTN_READ_REGISTER EQU PORTC

    GLOBAL  ReadButtons
ReadButtons

	clrf BUTTONS_STATUS

	bcf BTN_REGISTER,BTN_NLOAD ; Load button status into register
	bcf BTN_REGISTER,BTN_CLK ; CLK
	bsf BTN_REGISTER,BTN_CLK
	bsf BTN_REGISTER,BTN_NLOAD

	;bcf BTN_REGISTER,BTN_CLK ; CLK
	;bsf BTN_REGISTER,BTN_CLK
	btfsc BTN_READ_REGISTER,BTN_OUT ; If button status is 1
	bsf BUTTONS_STATUS,0 ; set corresponding bit to 1 in register.

	bcf BTN_REGISTER,BTN_CLK ; CLK
	bsf BTN_REGISTER,BTN_CLK
	btfsc BTN_READ_REGISTER,BTN_OUT ; If button status is 1
	bsf BUTTONS_STATUS,1 ; set corresponding bit to 1 in register.

	bcf BTN_REGISTER,BTN_CLK ; CLK
	bsf BTN_REGISTER,BTN_CLK
	btfsc BTN_READ_REGISTER,BTN_OUT ; If button status is 1
	bsf BUTTONS_STATUS,2 ; set corresponding bit to 1 in register.

	bcf BTN_REGISTER,BTN_CLK ; CLK
	bsf BTN_REGISTER,BTN_CLK
	btfsc BTN_READ_REGISTER,BTN_OUT ; If button status is 1
	bsf BUTTONS_STATUS,3 ; set corresponding bit to 1 in register.

	bcf BTN_REGISTER,BTN_CLK ; CLK
	bsf BTN_REGISTER,BTN_CLK
	btfsc BTN_READ_REGISTER,BTN_OUT ; If button status is 1
	bsf BUTTONS_STATUS,4 ; set corresponding bit to 1 in register.

	bcf BTN_REGISTER,BTN_CLK ; CLK
	bsf BTN_REGISTER,BTN_CLK
	btfsc BTN_READ_REGISTER,BTN_OUT ; If button status is 1
	bsf BUTTONS_STATUS,5 ; set corresponding bit to 1 in register.

	bcf BTN_REGISTER,BTN_CLK ; CLK
	bsf BTN_REGISTER,BTN_CLK
	btfsc BTN_READ_REGISTER,BTN_OUT ; If button status is 1
	bsf BUTTONS_STATUS,6 ; set corresponding bit to 1 in register.

	bcf BTN_REGISTER,BTN_CLK ; CLK
	bsf BTN_REGISTER,BTN_CLK
	btfsc BTN_READ_REGISTER,BTN_OUT ; If button status is 1
	bsf BUTTONS_STATUS,7 ; set corresponding bit to 1 in register.
	
	return

;******************************************************************************
;End of file

		END
