;******************************************************************************
;                                                                             *
;    Filename: tetris.asm                                                     *
;    Date:                                                                    *
;    File Version:                                                            *
;                                                                             *
;    Author:                                                                  *
;    Company:                                                                 *
;                                                                             * 
;******************************************************************************

	LIST P=18F2550, F=INHX32	;directive to define processor
	#include <P18F2550.INC>	;processor specific variable definitions

;******************************************************************************

	#include <main.inc>
;******************************************************************************

;Variable definitions

		UDATA_ACS
		
XREG   RES 1  ; Auxiliary "registers"
YREG   RES 1 
ZREG   RES 1 

RANDOM_COUNTER	RES 1

; Logic
; -----

GAME_OVER	    RES 1 ; 1 if Game Over else 0
GAME_PAUSED	    RES	1 ; 1 if pause, 0 if not
	    
TIMER_UPDATE_GAME   RES	1 ; Flag set to 1 by timer ISR and set by 0 by main loop.

FORCE_FALL_DOWN	    RES	1 ; Flag set to 1 when pressing UP, set to 0 when new form

FALL_DOWN_SPEED	    RES	1 ; Number of ticks to wait until next step
FALL_DOWN_REMAINING_STEPS   RES	1 ; Number of ticks elapsed
   
OLD_BUTTONS_STATUS	RES 1
NEWPRESS_BUTTONS_STATUS RES 1

OLD_BUTTONS_STATUS_STASEL	RES 1 ; Same as the previous two, for start and select
NEWPRESS_BUTTONS_STATUS_STASEL  RES 1

RIGHT_AUTOREPEAT_COUNT	RES 1
LEFT_AUTOREPEAT_COUNT	RES 1
DOWN_AUTOREPEAT_COUNT	RES 1
	
; Buffers
; -------

DRAW_FORM_TOGGLE    RES	1
    
SCORE_BUFFER	    RES	4 ; BCD (LSB first)
	    
; Fixed parts
FIXED_FRAMEBUFFER   RES 0x12 ; (=16+1+1) The last row is filled with ones as a wall.
			     ; The first one is filled with zeros always for the line
			     ; elimination algorithm

; Form information
; ----------------

; Buffer to mix fixed parts and moving parts
TETRIS_FRAMEBUFFER  RES 0x13 ; (=16+3) The last rows are used as a failsafe
			     ; for forms that are too short.

FORM_CURRENT	    RES 4 ; Current form (displaced 
FORM_CURRENT_NUMBER RES 1 ; Current form number (0-6)
FORM_CURRENT_ANGLE  RES 1 ; Current rotation (0-3)
FORM_DISPLACEMENT_X RES 1 ; Displacement from central position
FORM_DISPLACEMENT_Y RES 1 ; Displacement from top position
 
;******************************************************************************
	
	CODE ; let linker place main program

;******************************************************************************

    GLOBAL	TIMER_Tetris
TIMER_Tetris ; Called 61 times per second, should be FAST. Used to sync

	bsf TIMER_UPDATE_GAME,0 ; Allow main loop to update
	
	return

;******************************************************************************

	GLOBAL	SETUP_Tetris
SETUP_Tetris ; Called once at the beginning
	
	clrf	GAME_PAUSED
	clrf	GAME_OVER
	clrf	DRAW_FORM_TOGGLE
	;clrf	RANDOM_COUNTER ; Clear (or not?)
	
	call	ResetScore
	call	SendScoreToDisplays
	
	movlw	0x40 ; (=64)
	movwf	FALL_DOWN_SPEED
	movwf	FALL_DOWN_REMAINING_STEPS
	
	clrf	TIMER_UPDATE_GAME
	
	clrf	OLD_BUTTONS_STATUS
	clrf	NEWPRESS_BUTTONS_STATUS
	clrf	OLD_BUTTONS_STATUS_STASEL
	clrf	NEWPRESS_BUTTONS_STATUS_STASEL
	clrf	RIGHT_AUTOREPEAT_COUNT
	clrf	LEFT_AUTOREPEAT_COUNT
	clrf	DOWN_AUTOREPEAT_COUNT
	
	;Clear frame
	movlw 0x11 ; (=16+1)
	lfsr FSR0,FIXED_FRAMEBUFFER+1 ; start off by one
ClearMore1
	clrf POSTINC0	 	; FIXED_FRAMEBUFFER[FSR0++] = 0
	decfsz WREG,W
	bra ClearMore1
	movlw	0xFF
	movwf	FIXED_FRAMEBUFFER+0 ; first row is a fixed wall
	
	movlw 0x10 ; (=16)
	lfsr FSR0,TETRIS_FRAMEBUFFER
ClearMore2
	clrf POSTINC0	 	; TETRIS_FRAMEBUFFER[FSR0++] = 0
	decfsz WREG,W
	bra ClearMore2

	call	GenerateForm ; This checks if collision, so don't use it until
	; wall buffers are cleared (FIXED_FRAMEBUFFER).

	return

;******************************************************************************

FORMS_DATA ; Each form has 4 bytes per rotation = 4*4 = 16 bytes of data
	; Data (4 bytes)  = FORMS_DATA + 16*Number_of_form + 4*Rotation
	#include <forms_data.inc>

GetCurrentFormData
	
	lfsr	FSR0,FORMS_DATA
	movf	FORM_CURRENT_NUMBER,W ; form number (3 bits)
	rlncf	WREG,W
	rlncf	WREG,W ; multiply by 4
	iorwf	FORM_CURRENT_ANGLE,W ; add rotation (3+2 bits)
	rlncf	WREG,W
	rlncf	WREG,W
	rlncf	WREG,W ; multiply by 8 again because each one is 4*2 bytes (3+2+3 bits)
	addlw	low FORMS_DATA
	movwf	TBLPTRL
	
	movlw	high FORMS_DATA
	bnc	NotCarry
	addlw	1
NotCarry
	movwf	TBLPTRH
	
	movlw	upper FORMS_DATA
	bnc	NotCarry2
	addlw	1
NotCarry2
	movwf	TBLPTRU
	
	TBLRD*+
	movff	TABLAT,FORM_CURRENT+0
	TBLRD*+
	TBLRD*+
	movff	TABLAT,FORM_CURRENT+1
	TBLRD*+
	TBLRD*+
	movff	TABLAT,FORM_CURRENT+2
	TBLRD*+
	TBLRD*
	movff	TABLAT,FORM_CURRENT+3
	   
	; Now move left or right to match last position
	
	tstfsz	FORM_DISPLACEMENT_X
	bra	DontEndDisplacement
	bra	EndDisplacement ; skip this if form wasn't moved
DontEndDisplacement
	
	movff	FORM_DISPLACEMENT_X,YREG ; Save current displacement here
	clrf	FORM_DISPLACEMENT_X
	
	btfss YREG,7 ; check if negative
	bra PositiveX
	; Negative
MoveLeft
	call	FormMoveLeftIgnoreFixedForms
	btfss	WREG,0
	bra	EndDisplacement ; can't move more

	incf	YREG,F ; if reached position previous to rotation
	bnz	MoveLeft

	bra	EndDisplacement
PositiveX
	; Positive
	
MoveRight
	call	FormMoveRightIgnoreFixedForms
	btfss	WREG,0
	bra	EndDisplacement ; can't move more
		
	decf	YREG,F ; if reached position previous to rotation
	bnz	MoveRight
	
EndDisplacement
	
	return

;******************************************************************************

FormMoveRightIgnoreFixedForms
	
	btfsc	FORM_CURRENT+0,0
	retlw 0
	btfsc	FORM_CURRENT+1,0
	retlw 0
	btfsc	FORM_CURRENT+2,0
	retlw 0
	btfsc	FORM_CURRENT+3,0
	retlw 0
	
	rrncf	FORM_CURRENT+0,F
	rrncf	FORM_CURRENT+1,F
	rrncf	FORM_CURRENT+2,F
	rrncf	FORM_CURRENT+3,F
	incf	FORM_DISPLACEMENT_X,F
	
	retlw 1

FormMoveLeftIgnoreFixedForms
	
	btfsc	FORM_CURRENT+0,7
	retlw 0
	btfsc	FORM_CURRENT+1,7
	retlw 0
	btfsc	FORM_CURRENT+2,7
	retlw 0
	btfsc	FORM_CURRENT+3,7
	retlw 0
	
	rlncf	FORM_CURRENT+0,F
	rlncf	FORM_CURRENT+1,F
	rlncf	FORM_CURRENT+2,F
	rlncf	FORM_CURRENT+3,F
	decf	FORM_DISPLACEMENT_X,F

	retlw 1

;******************************************************************************
	
FormMoveRight
	
	btfsc	FORM_CURRENT+0,0
	retlw 0
	btfsc	FORM_CURRENT+1,0
	retlw 0
	btfsc	FORM_CURRENT+2,0
	retlw 0
	btfsc	FORM_CURRENT+3,0
	retlw 0
	
	rrncf	FORM_CURRENT+0,F
	rrncf	FORM_CURRENT+1,F
	rrncf	FORM_CURRENT+2,F
	rrncf	FORM_CURRENT+3,F
	incf	FORM_DISPLACEMENT_X,F
	
	rcall	CheckCollisionFormTetrisBuffer
	tstfsz	WREG
	bra	CollisionMoveRight
	retlw 1
	
CollisionMoveRight
	rlncf	FORM_CURRENT+0,F
	rlncf	FORM_CURRENT+1,F
	rlncf	FORM_CURRENT+2,F
	rlncf	FORM_CURRENT+3,F
	decf	FORM_DISPLACEMENT_X,F
	retlw	0
	
;******************************************************************************
	
FormMoveLeft
	btfsc	FORM_CURRENT+0,7
	retlw 0
	btfsc	FORM_CURRENT+1,7
	retlw 0
	btfsc	FORM_CURRENT+2,7
	retlw 0
	btfsc	FORM_CURRENT+3,7
	retlw 0
	
	rlncf	FORM_CURRENT+0,F
	rlncf	FORM_CURRENT+1,F
	rlncf	FORM_CURRENT+2,F
	rlncf	FORM_CURRENT+3,F
	decf	FORM_DISPLACEMENT_X,F
	
	rcall	CheckCollisionFormTetrisBuffer
	tstfsz	WREG
	bra	CollisionMoveLeft
	retlw 1
	
CollisionMoveLeft
	rrncf	FORM_CURRENT+0,F
	rrncf	FORM_CURRENT+1,F
	rrncf	FORM_CURRENT+2,F
	rrncf	FORM_CURRENT+3,F
	incf	FORM_DISPLACEMENT_X,F
	retlw	0

;******************************************************************************

FormRotateRight
	
	movff	FORM_DISPLACEMENT_X,ZREG
	
	incf	FORM_CURRENT_ANGLE,W
	andlw	0x03
	movwf	FORM_CURRENT_ANGLE
	
	rcall	GetCurrentFormData
	
	rcall	CheckCollisionFormTetrisBuffer
	tstfsz	WREG
	bra	CollisionRotateRight
	
	return
	
CollisionRotateRight
	
	; Try moving left and right
	
	movf	ZREG,W
	incf	WREG ; right
	movwf	FORM_DISPLACEMENT_X

	rcall	GetCurrentFormData
	
	rcall	CheckCollisionFormTetrisBuffer
	tstfsz	WREG
	bra	ColRotateRightFirstTryFailed
	
	return ; OK
	
ColRotateRightFirstTryFailed
	
	movf	ZREG,W
	decf	WREG ; left
	movwf	FORM_DISPLACEMENT_X
	
	rcall	GetCurrentFormData
	
	rcall	CheckCollisionFormTetrisBuffer
	tstfsz	WREG
	bra	ColRotateRightSecondTryFailed
	
	return ; OK
	
ColRotateRightSecondTryFailed
	
	tstfsz	FORM_CURRENT_NUMBER ; if stick, try left 2 times
	bra	ColRotateRightAllTriesFailed
	
	movf	ZREG,W
	decf	WREG ; left
	decf	WREG ; left
	movwf	FORM_DISPLACEMENT_X
	
	rcall	GetCurrentFormData
	
	rcall	CheckCollisionFormTetrisBuffer
	tstfsz	WREG
	bra	ColRotateRightAllTriesFailed
	
	return ; OK
	
ColRotateRightAllTriesFailed
	
	movff	ZREG,FORM_DISPLACEMENT_X
	decf	FORM_CURRENT_ANGLE,W
	andlw	0x03
	movwf	FORM_CURRENT_ANGLE
	
	rcall	GetCurrentFormData
	
	return
	
;******************************************************************************

FormRotateLeft
	
	movff	FORM_DISPLACEMENT_X,ZREG
	
	decf	FORM_CURRENT_ANGLE,W
	andlw	0x03
	movwf	FORM_CURRENT_ANGLE
	
	rcall	GetCurrentFormData
	
	rcall	CheckCollisionFormTetrisBuffer
	tstfsz	WREG
	bra	CollisionRotateLeft
	
	return
	
CollisionRotateLeft
	
	; Try moving left and right
	
	movf	ZREG,W
	incf	WREG ; right
	movwf	FORM_DISPLACEMENT_X

	rcall	GetCurrentFormData
	
	rcall	CheckCollisionFormTetrisBuffer
	tstfsz	WREG
	bra	ColRotateLeftFirstTryFailed
	
	return ; OK
	
ColRotateLeftFirstTryFailed
	
	movf	ZREG,W
	decf	WREG ; left
	movwf	FORM_DISPLACEMENT_X
	
	rcall	GetCurrentFormData
	
	rcall	CheckCollisionFormTetrisBuffer
	tstfsz	WREG
	bra	ColRotateLeftSecondTryFailed
	
	return ; OK
	
ColRotateLeftSecondTryFailed
	
	tstfsz	FORM_CURRENT_NUMBER ; if stick, try left 2 times
	bra	ColRotateLeftAllTriesFailed
	
	movf	ZREG,W
	decf	WREG ; left
	decf	WREG ; left
	movwf	FORM_DISPLACEMENT_X
	
	rcall	GetCurrentFormData
	
	rcall	CheckCollisionFormTetrisBuffer
	tstfsz	WREG
	bra	ColRotateLeftAllTriesFailed
	
	return ; OK
	
ColRotateLeftAllTriesFailed
	
	movff	ZREG,FORM_DISPLACEMENT_X
	incf	FORM_CURRENT_ANGLE,W
	andlw	0x03
	movwf	FORM_CURRENT_ANGLE
	
	rcall	GetCurrentFormData
	
	return
	
;******************************************************************************

DrawFormTetrisBuffer
	
	movf	FORM_DISPLACEMENT_Y,W
	negf	WREG
	addlw	0xC ; (=12) Transform coordinate from top position inside framebuffer
	
	btfsc	WREG,7 ; check if negative
	bra	ThisIsNegative
	
	; Positive
	addlw	low TETRIS_FRAMEBUFFER
	movwf	FSR0L
	movlw	high TETRIS_FRAMEBUFFER
	bnc NotCarryHere
	incf	WREG
NotCarryHere
	movwf	FSR0H

	bra	DrawNow
ThisIsNegative
	
	addlw	low TETRIS_FRAMEBUFFER
	movwf	FSR0L
	movlw	high TETRIS_FRAMEBUFFER
	bc	NotBorrowHere
	decf	WREG
NotBorrowHere
	movwf	FSR0H
	
DrawNow
	movf	FORM_CURRENT+3,W
	iorwf	INDF0,W
	movwf	POSTINC0
	
	movf	FORM_CURRENT+2,W
	iorwf	INDF0,W
	movwf	POSTINC0
	
	movf	FORM_CURRENT+1,W
	iorwf	INDF0,W
	movwf	POSTINC0
	
	movf	FORM_CURRENT+0,W
	iorwf	INDF0,W
	movwf	INDF0

	return
	
;******************************************************************************
	
ClearLines
	
	clrf	XREG ; Number of lines eliminated this time
	
	lfsr	FSR0,FIXED_FRAMEBUFFER+1

LoopLines
	incf	INDF0,W
	bz	ThisLineIsFull
	decf	WREG,W
	bz	ThisLineIsEmpty

Continue
	movf	POSTINC0,W
	
	bra	LoopLines
	
ThisLineIsFull
	
	incf	XREG,F ; Increase number of rows eliminated
	; Eliminate this row, move the rest
	
	movff	FSR0L,YREG
	movff	FSR0H,ZREG
	
	; Use FSR2 and FSR0 to iterate
	
	movff	FSR0L,FSR2L
	movff	FSR0H,FSR2H
	
	movf	POSTINC2,W ; increment FSR2
	
MoveNextLine
	
	movf	POSTINC2,W
	movwf	POSTINC0
	
	tstfsz	WREG ; if zero, stop
	bra	MoveNextLine
	
	; Restore FSR0 to its previous value
	
	movff	YREG,FSR0L
	movff	ZREG,FSR0H
	movf	POSTDEC0,W ; Check the previous one in case it's full too
	
	bra Continue
	
ThisLineIsEmpty ; exit
	
	movf	XREG,W ; return number of lines eliminated in WREG
	return
	
;******************************************************************************
	
FixFormGenerateNew
	
	call	DrawFormTetrisBuffer ; draw intro tetris buffer
	
	; copy tetris buffer into fixed buffer
	
	lfsr	FSR2,TETRIS_FRAMEBUFFER
	lfsr	FSR0,FIXED_FRAMEBUFFER+1
	
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	
	; Generate new form, but check lines before
	
	call	ClearLines
	andwf	WREG,W
	bz	NoLinesEliminatedNow
	; WREG is not 0, add to score
	call	AddLinesToScore
	call	SendScoreToDisplays
NoLinesEliminatedNow
	
	call	GenerateForm
	
	return

;******************************************************************************

GenerateForm
	
	clrf	FORCE_FALL_DOWN
	
	clrf	FORM_DISPLACEMENT_X
	clrf	FORM_DISPLACEMENT_Y

	clrf	FORM_CURRENT_ANGLE

	movf	RANDOM_COUNTER,W
	andlw	0x07
	movwf	FORM_CURRENT_NUMBER
	
	sublw	0x07
	bnz	ExitGenerateForm ; 7 doesn't exist, try again
	
	movf	RANDOM_COUNTER,W
	rrncf	WREG
	rrncf	WREG
	rrncf	WREG
	andlw	0x07
	movwf	FORM_CURRENT_NUMBER
	
	sublw	0x07
	bnz	ExitGenerateForm ; 7 doesn't exist, set to 0 and let it be...
	
	clrf	FORM_CURRENT_NUMBER
	
ExitGenerateForm
	rcall	GetCurrentFormData

	rcall	CheckCollisionFormTetrisBuffer
	tstfsz	WREG
	bra	DetectedGameOver
	
	return

DetectedGameOver
	bsf	GAME_OVER,0
	return
	
;******************************************************************************
	
MoveDown
	
	; It doesn't matter how we got there, set the automatic countdown
	; to full time.
	movf	FALL_DOWN_SPEED,W
	movwf	FALL_DOWN_REMAINING_STEPS
	
	incf	FORM_DISPLACEMENT_Y,F
	call	CheckCollisionFormTetrisBuffer
	
	tstfsz	WREG
	bra	Collision
	
	retlw	0 ; Didn't generate a new form
	
Collision ; go back and fix
	decf	FORM_DISPLACEMENT_Y,F
	
	call	FixFormGenerateNew
	
	retlw	1 ; New form generated
	
;******************************************************************************

CheckCollisionFormTetrisBuffer
	
	movf	FORM_DISPLACEMENT_Y,W
	negf	WREG
	addlw	0xD ; (=12+1) Transform coordinate from top position inside framebuffer
	
	btfsc	WREG,7 ; check if negative
	bra	ThisIsNegative_
	
	; Positive
	addlw	low (FIXED_FRAMEBUFFER+0)
	movwf	FSR0L
	movlw	high (FIXED_FRAMEBUFFER+0)
	bnc NotCarryHere_
	incf	WREG
NotCarryHere_
	movwf	FSR0H

	bra	DrawNow_
ThisIsNegative_
	
	addlw	low (FIXED_FRAMEBUFFER+0)
	movwf	FSR0L
	movlw	high (FIXED_FRAMEBUFFER+0)
	bc	NotBorrowHere_
	decf	WREG
NotBorrowHere_
	movwf	FSR0H
	
DrawNow_
	
	clrf	XREG
	
	movf	FORM_CURRENT+3,W
	andwf	POSTINC0,W
	iorwf	XREG,F
	
	movf	FORM_CURRENT+2,W
	andwf	POSTINC0,W
	iorwf	XREG,F
	
	movf	FORM_CURRENT+1,W
	andwf	POSTINC0,W
	iorwf	XREG,F
	
	movf	FORM_CURRENT+0,W
	andwf	INDF0,W
	iorwf	XREG,F
	
	tstfsz	XREG
	retlw	1 ; collision
	
	retlw	0 ; no collision
	
;******************************************************************************

BCDto7SEG
	incf	WREG,W
	
	decf	WREG,W
	bz	BCD_0
	decf	WREG,W
	bz	BCD_1
	decf	WREG,W
	bz	BCD_2
	decf	WREG,W
	bz	BCD_3
	decf	WREG,W
	bz	BCD_4
	decf	WREG,W
	bz	BCD_5
	decf	WREG,W
	bz	BCD_6
	decf	WREG,W
	bz	BCD_7
	decf	WREG,W
	bz	BCD_8
	decf	WREG,W
	bz	BCD_9
	bra	BCD_UNK
	
BCD_0	retlw	B'11111100'
BCD_1	retlw	B'01100000'
BCD_2	retlw	B'11011010'
BCD_3	retlw	B'11110010'
BCD_4	retlw	B'01100110'
BCD_5	retlw	B'10110110'
BCD_6	retlw	B'10111110'
BCD_7	retlw	B'11100000'
BCD_8	retlw	B'11111110'
BCD_9	retlw	B'11110110'
BCD_UNK	retlw	B'00000001'
	
;******************************************************************************

SendScoreToDisplays
	movf	SCORE_BUFFER+0,W
	call	BCDto7SEG
	movwf	SEVENSEG_FRAMEBUFFER+3
	
	movf	SCORE_BUFFER+1,W
	call	BCDto7SEG
	movwf	SEVENSEG_FRAMEBUFFER+2
	
	movf	SCORE_BUFFER+2,W
	call	BCDto7SEG
	movwf	SEVENSEG_FRAMEBUFFER+1
	
	movf	SCORE_BUFFER+3,W
	call	BCDto7SEG
	movwf	SEVENSEG_FRAMEBUFFER+0
	
	return
	
ResetScore
	clrf	WREG
	movwf	SCORE_BUFFER+0
	movwf	SCORE_BUFFER+1
	movwf	SCORE_BUFFER+2
	movwf	SCORE_BUFFER+3
	return

LinesToScore ; W shouldn't be 0 (1 to 4 is ok)
	decf	WREG,W
	bz	L2S_1
	decf	WREG,W
	bz	L2S_2
	decf	WREG,W
	bz	L2S_3
	decf	WREG,W
	bz	L2S_4
	bra	L2S_UNK ; return 0
L2S_1	retlw	1
L2S_2	retlw	4
L2S_3	retlw	9
L2S_4	retlw	0x10 ; (=16)
L2S_UNK	retlw	0
	
AddLinesToScore ; WREG = lines
	call	LinesToScore
	movwf	XREG ; save in XREG the number of points to add
	
AddMoreToScore
	incf	SCORE_BUFFER+0,F
	movlw	0x0A ; (=10)
	cpfseq	SCORE_BUFFER+0
	bra	IncrementEnd
	clrf	SCORE_BUFFER+0
	
	incf	SCORE_BUFFER+1,F
	movlw	0x0A ; (=10)
	cpfseq	SCORE_BUFFER+1
	bra	IncrementEnd
	clrf	SCORE_BUFFER+1
	
	incf	SCORE_BUFFER+2,F
	movlw	0x0A ; (=10)
	cpfseq	SCORE_BUFFER+2
	bra	IncrementEnd
	clrf	SCORE_BUFFER+2
	
	; For each 100 points, increase speed
	decf	FALL_DOWN_SPEED,F
	movlw	0x03
	cpfseq	FALL_DOWN_SPEED
	bra	NotReachedLimit
	movlw	0x04 ; limit = 4
	movwf	FALL_DOWN_SPEED
NotReachedLimit
	
	incf	SCORE_BUFFER+3,F
	movlw	0x0A ; (=10)
	cpfseq	SCORE_BUFFER+3
	bra	IncrementEnd
	clrf	SCORE_BUFFER+3
	
IncrementEnd
	decf	XREG,F
	bnz	AddMoreToScore
	
	return
	
;******************************************************************************

    GLOBAL	MAIN_Tetris
MAIN_Tetris ; Shouldn't return (it will reset the controller)
	
	incf	RANDOM_COUNTER,F
	
	btfss	TIMER_UPDATE_GAME,0
	bra MAIN_Tetris ; Wait until the timer sets the enable flag
	bcf TIMER_UPDATE_GAME,0 ; Clear flag

	; Read buttons

	call	ReadButtons
	
	movf	BUTTONS_STATUS,W
	xorwf	OLD_BUTTONS_STATUS_STASEL,W
	andwf	BUTTONS_STATUS,W
	movwf	NEWPRESS_BUTTONS_STATUS_STASEL
	movff	BUTTONS_STATUS,OLD_BUTTONS_STATUS_STASEL
	
	; START and SELECT buttons are handled here...
	
	btfsc	NEWPRESS_BUTTONS_STATUS_STASEL,BIT_PAD_SELECT
	reset
		
	btfsc	NEWPRESS_BUTTONS_STATUS_STASEL,BIT_PAD_START
	btg	GAME_PAUSED,0
	
	; Check Game Over
	
	btfss	GAME_OVER,0
	bra	NotOver
	
	bsf	SEVENSEG_FRAMEBUFFER+0,0
	bsf	SEVENSEG_FRAMEBUFFER+1,0
	bsf	SEVENSEG_FRAMEBUFFER+2,0
	bsf	SEVENSEG_FRAMEBUFFER+3,0 ; Draw dots
	
	bra	IgnoreButtons
NotOver
	
	bcf	SEVENSEG_FRAMEBUFFER+0,0
	bcf	SEVENSEG_FRAMEBUFFER+1,0
	bcf	SEVENSEG_FRAMEBUFFER+2,0
	bcf	SEVENSEG_FRAMEBUFFER+3,0 ; Clear dots
	
	; Check Pause
	
	btfss	GAME_PAUSED,0
	bra	NotPaused
	
	bsf	SEVENSEG_FRAMEBUFFER+0,0
	bsf	SEVENSEG_FRAMEBUFFER+3,0 ; Draw dots
	
	bra	IgnoreButtons
NotPaused
	
	bcf	SEVENSEG_FRAMEBUFFER+0,0
	bcf	SEVENSEG_FRAMEBUFFER+3,0 ; Clear dots
	
	; If there are lines to eliminate, skip logic (jump after button handling)

	call	ClearLines
	andwf	WREG,W
	bz	NoLinesEliminated
	; WREG is not 0, add to score
	call	AddLinesToScore
	call	SendScoreToDisplays
	bra	IgnoreButtons
	
NoLinesEliminated
	
	; Check if form has to go down. If so, ignore buttons this frame
	
;	btfss	NEWPRESS_BUTTONS_STATUS,BIT_PAD_DOWN
;	bra	NotPressedDown
;	call	MoveDown
;	bcf	NEWPRESS_BUTTONS_STATUS,BIT_PAD_DOWN ; if not it will hang falling
;	bra	IgnoreButtons
;NotPressedDown

	btfss	FORCE_FALL_DOWN,0
	bra	NotForceFall
	call	MoveDown
	bra	IgnoreButtons
NotForceFall
	
	tstfsz	FALL_DOWN_REMAINING_STEPS
	bra	NotAutomaticFall
	movf	FALL_DOWN_SPEED,W
	movwf	FALL_DOWN_REMAINING_STEPS
	call	MoveDown
	bra	IgnoreButtons
NotAutomaticFall
	decf	FALL_DOWN_REMAINING_STEPS,F
	
	; Update button status
	
	call	ButtonsHandleInGame
IgnoreButtons
	
	; Update temporal framebuffer from fixed framebuffer

	lfsr	FSR0,TETRIS_FRAMEBUFFER
	lfsr	FSR2,FIXED_FRAMEBUFFER+1
	
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	
	; Draw moving piece into temporal framebuffer
	
	btg	DRAW_FORM_TOGGLE,0
	btfss	DRAW_FORM_TOGGLE,0
	call	DrawFormTetrisBuffer
	
	; Update framebuffer
	
	lfsr	FSR0,MATRIX_FRAMEBUFFER
	lfsr	FSR2,TETRIS_FRAMEBUFFER
	
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
	movff	POSTINC2,POSTINC0
		
	bra MAIN_Tetris
	
	return

;******************************************************************************    

ButtonsHandleInGame
	
	movf	BUTTONS_STATUS,W
	xorwf	OLD_BUTTONS_STATUS,W
	andwf	BUTTONS_STATUS,W
	movwf	NEWPRESS_BUTTONS_STATUS
	movff	BUTTONS_STATUS,OLD_BUTTONS_STATUS
	
	; Right Autorepeat
	
	btfss	BUTTONS_STATUS,BIT_PAD_RIGHT
	bra	RightNotPressed
	; Right pressed
	incf	RIGHT_AUTOREPEAT_COUNT,F
	btfss	RIGHT_AUTOREPEAT_COUNT,4
	bra	EndRightAutorepeat
	btfss	RIGHT_AUTOREPEAT_COUNT,2
	bra	EndRightAutorepeat
	; Reached 16 - Step every 4 frames
	movlw	0x10
	movwf	RIGHT_AUTOREPEAT_COUNT
	bsf	NEWPRESS_BUTTONS_STATUS,BIT_PAD_RIGHT
	bra	EndRightAutorepeat
RightNotPressed
	; Right not pressed
	clrf	RIGHT_AUTOREPEAT_COUNT
EndRightAutorepeat
	
	; Left Autorepeat
	
	btfss	BUTTONS_STATUS,BIT_PAD_LEFT
	bra	LeftNotPressed
	; Left pressed
	incf	LEFT_AUTOREPEAT_COUNT,F
	btfss	LEFT_AUTOREPEAT_COUNT,4
	bra	EndLeftAutorepeat
	btfss	LEFT_AUTOREPEAT_COUNT,2
	bra	EndLeftAutorepeat
	; Reached 16 - Step every 4 frames
	movlw	0x10
	movwf	LEFT_AUTOREPEAT_COUNT
	bsf	NEWPRESS_BUTTONS_STATUS,BIT_PAD_LEFT
	bra	EndLeftAutorepeat
LeftNotPressed
	; Left not pressed
	clrf	LEFT_AUTOREPEAT_COUNT
EndLeftAutorepeat
	
	; Down Autorepeat
	
	btfss	BUTTONS_STATUS,BIT_PAD_DOWN
	bra	DownNotPressed
	; Down pressed
	incf	DOWN_AUTOREPEAT_COUNT,F
	btfss	DOWN_AUTOREPEAT_COUNT,4
	bra	EndDownAutorepeat
	btfss	DOWN_AUTOREPEAT_COUNT,2
	bra	EndDownAutorepeat
	; Reached 16 - Step every 4 frames
	movlw	0x10
	movwf	DOWN_AUTOREPEAT_COUNT
	bsf	NEWPRESS_BUTTONS_STATUS,BIT_PAD_DOWN
	bra	EndDownAutorepeat
DownNotPressed
	; Down not pressed
	clrf	DOWN_AUTOREPEAT_COUNT
EndDownAutorepeat
	
	; Normal button handling
	
	btfsc	NEWPRESS_BUTTONS_STATUS,BIT_PAD_RIGHT
	rcall	FormMoveRight
	btfsc	NEWPRESS_BUTTONS_STATUS,BIT_PAD_LEFT
	rcall	FormMoveLeft
	
	btfsc	NEWPRESS_BUTTONS_STATUS,BIT_PAD_UP
	bsf	FORCE_FALL_DOWN,0
	btfsc	NEWPRESS_BUTTONS_STATUS,BIT_PAD_DOWN
	call	MoveDown ; Done in main loop

	btfsc	NEWPRESS_BUTTONS_STATUS,BIT_PAD_A
	call	FormRotateRight
	btfsc	NEWPRESS_BUTTONS_STATUS,BIT_PAD_B
	call	FormRotateLeft
		
	return
	
;******************************************************************************
    END
